"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const tslib_1 = require("tslib");
const devkit_1 = require("@nx/devkit");
const child_process_1 = require("child_process");
function getCommandFromTarget(project, targetName, commandGrep) {
    const target = project.targets[targetName];
    if (!target) {
        throw new Error(`Could not find target '${targetName}' in project '${project.name}'`);
    }
    const commands = [
        ...(target.options.command ? [target.options.command] : []),
        ...(target.options.commands ? target.options.commands : []),
    ].filter((cmd) => cmd.includes(commandGrep));
    if (commands.length === 0) {
        throw new Error(`Could not find a command in target '${targetName}' that matches '${commandGrep}'`);
    }
    if (commands.length !== 1) {
        devkit_1.logger.warn(`Found multiple commands in target '${targetName}' that match '${commandGrep}', using first match`);
    }
    return commands[0];
}
function runFirebaseServeExecutor(options, context) {
    return tslib_1.__awaiter(this, void 0, void 0, function* () {
        return new Promise((resolve) => {
            const projectName = context.projectName;
            const projects = context.projectsConfigurations;
            const project = projects.projects[projectName];
            // Determine the watch target command
            const watchCommand = getCommandFromTarget(project, 'watch', 'nx run-many --targets=build');
            // Determine the firebase target command, so we get --config & --project
            const firebaseCommand = getCommandFromTarget(project, 'firebase', 'firebase');
            // Determine the emulator target command
            const emulateCommand = getCommandFromTarget(project, 'emulate', 'emulators:').replace(`nx run ${projectName}:firebase`, '');
            // Run the watch process
            // eslint-disable-next-line
            const watchProcess = (0, child_process_1.spawn)(watchCommand, [], {
                shell: true,
                stdio: 'inherit',
                detached: false,
            }).on('exit', (code) => {
                if (!code) {
                    devkit_1.logger.warn(`serve: watch process finished successfully`);
                }
                else {
                    devkit_1.logger.error(`serve: watch process finished with error '${code}'`);
                }
            });
            // determine any extra commands passed on the command line
            const extraArgs = options.__unparsed__.length > 0
                ? ' ' + options.__unparsed__.join(' ')
                : '';
            // Run the firebase emulator process
            const emulatorProcess = (0, child_process_1.spawn)(`${firebaseCommand} ${emulateCommand} ${extraArgs}`, [], {
                shell: true,
                stdio: 'inherit',
                detached: false,
            });
            emulatorProcess.on('exit', (code) => {
                if (!code) {
                    devkit_1.logger.warn(`serve: Firebase emulator finished successfully`);
                    resolve({ success: true }); // not sure what difference this makes
                }
                else {
                    devkit_1.logger.error(`serve: Firebase emulator finished with error '${code}'`);
                    resolve({ success: false });
                }
            });
            // Handle signals for serve executor process
            const processExitListener = (signal) => {
                // logger.error(`\nserve: executor received signal '${signal}'`)
                if (signal === 'SIGINT') {
                    devkit_1.logger.warn(`\nserve: terminating`);
                    // no need for these it seems
                    // emulatorProcess.kill()
                    // watchProcess.kill()
                }
                if (signal === 0) {
                    devkit_1.logger.warn(`serve: finished successfully`);
                }
                if (signal === 1) {
                    devkit_1.logger.error(`serve: finished with errors`);
                }
            };
            process.on('exit', processExitListener);
            process.on('SIGTERM', processExitListener);
            process.on('SIGINT', processExitListener);
        });
    });
}
exports.default = runFirebaseServeExecutor;
//# sourceMappingURL=serve.js.map