"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const tslib_1 = require("tslib");
const devkit_1 = require("@nx/devkit");
const init_1 = require("../init/init");
const lib_1 = require("../sync/lib");
const fs_1 = require("fs");
/**
 * Migrate firebase workspace
 *
 */
function migrateGenerator(tree, 
// eslint-disable-next-line
options) {
    return tslib_1.__awaiter(this, void 0, void 0, function* () {
        const tasks = [];
        // initialise plugin
        const initTask = yield (0, init_1.default)(tree, {});
        tasks.push(initTask);
        // otherwise, sync the workspace.
        // build lists of firebase apps & functions that have been deleted or renamed
        (0, lib_1.debugInfo)('- Migrating workspace');
        const workspace = (0, lib_1.getFirebaseWorkspace)(tree);
        devkit_1.logger.info(`This workspace has ${workspace.firebaseAppProjects.size} firebase apps and ${workspace.firebaseFunctionProjects.size} firebase functions\n\n`);
        devkit_1.logger.info(`Running plugin migrations for workspace`);
        // ensure ignores in .gitignore
        // ensure ignores in .nxignore
        // init generator takes care of this
        // [2.0.0 -> 2.1.0] ensure environment files are present in apps dir
        workspace.firebaseAppProjects.forEach((project, name) => {
            const envPath = `${project.root}/environment`;
            const envFiles = [`.env`, `.env.local`, `.secret.local`];
            for (const envFile of envFiles) {
                const fullPath = `${envPath}/${envFile}`;
                if (!tree.exists(fullPath)) {
                    const src = (0, fs_1.readFileSync)((0, devkit_1.joinPathFragments)(__dirname, '..', 'application', 'files', 'environment', envFile));
                    tree.write((0, devkit_1.joinPathFragments)(project.root, 'environment', envFile), src);
                    devkit_1.logger.info(`MIGRATE Added default environment file 'environment/${envFile}' for firebase app '${name}'`);
                }
            }
        });
        // [2.0.0 -> 2.1.0] ensure getconfig path is environment
        workspace.firebaseAppProjects.forEach((project, name) => {
            const getconfig = project.targets['getconfig'];
            const command = getconfig === null || getconfig === void 0 ? void 0 : getconfig.options.command;
            const legacyPath = (0, devkit_1.joinPathFragments)(project.root, '.runtimeconfig.json');
            if (command.includes(legacyPath)) {
                getconfig.options.command = command.replace(legacyPath, (0, devkit_1.joinPathFragments)(project.root, 'environment', '.runtimeconfig.json'));
                devkit_1.logger.info(`MIGRATE Updated getconfig target to use ignore environment directory for firebase app '${name}'`);
                (0, devkit_1.updateProjectConfiguration)(tree, project.name, project);
            }
        });
        // [2.0.0 -> 2.1.0] ensure globs in function projects
        workspace.firebaseFunctionProjects.forEach((project, name) => {
            var _a;
            const appName = (0, lib_1.getFirebaseScopeFromTag)(project, 'firebase:dep');
            const appProject = workspace.firebaseAppProjects.get(appName.tagValue);
            const assets = (_a = project.targets['build']) === null || _a === void 0 ? void 0 : _a.options.assets;
            const globs = {
                glob: '**/*',
                input: `${appProject.root}/environment`,
                output: '.',
            };
            let hasAssetsGlob = false;
            assets.forEach((asset) => {
                if (typeof asset === 'object') {
                    if (asset.glob === globs.glob &&
                        asset.input === globs.input &&
                        asset.output === globs.output) {
                        hasAssetsGlob = true;
                    }
                }
            });
            if (!hasAssetsGlob) {
                assets.push(globs);
                devkit_1.logger.info(`MIGRATE Added assets glob for firebase function app '${name}'`);
                (0, devkit_1.updateProjectConfiguration)(tree, project.name, project);
            }
        });
        // [2.0.0 -> 2.1.0] ensure ignores to function firebase.json
        workspace.firebaseConfigs.forEach((config, configFilename) => {
            if (!Array.isArray(config.functions)) {
                // promote to array if single function object
                config.functions = [config.functions];
            }
            config.functions.map((func) => {
                const ignoreRule = '*.local';
                const ignore = func.ignore || [];
                if (!ignore.includes(ignoreRule)) {
                    devkit_1.logger.info(`MIGRATE Added ignore rule to firebase config '${configFilename}' for firebase function codebase '${func.codebase}'`);
                    ignore.push(ignoreRule);
                }
                func.ignore = ignore;
            });
            (0, devkit_1.writeJson)(tree, configFilename, config);
        });
        // [2.0.0 -> 2.1.0] change firebase serve target
        workspace.firebaseAppProjects.forEach((project, name) => {
            const serve = project.targets['serve'];
            const serveExecutor = '@simondotm/nx-firebase:serve';
            if (serve.executor !== serveExecutor) {
                serve.executor = serveExecutor;
                devkit_1.logger.info(`MIGRATE Updated serve target for firebase app '${name}'`);
                (0, devkit_1.updateProjectConfiguration)(tree, project.name, project);
            }
        });
        return (0, devkit_1.runTasksInSerial)(...tasks);
    });
}
exports.default = migrateGenerator;
//# sourceMappingURL=migrate.js.map