"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.getFirebaseChanges = void 0;
const utils_1 = require("../../../utils");
const tags_1 = require("./tags");
/**
 * Check if the given firebase project has been renamed by checking its `firebase:name` tag
 * @param project - Firebase project to be checked
 * @returns the previous project name if renamed, otherwise undefined/falsy
 */
function isRenamed(project) {
    const { tagValue } = (0, tags_1.getFirebaseScopeFromTag)(project, 'firebase:name');
    (0, utils_1.debugInfo)(`- checking if project ${project.name} is renamed, will be different to ${tagValue}`);
    if (tagValue !== project.name) {
        (0, utils_1.debugInfo)(`- firebase project ${tagValue} has been renamed to ${project.name} `);
        return tagValue;
    }
    return undefined;
}
function getFirebaseChanges(tree, projects, configs) {
    // map of project name -> deletion status
    const deletedApps = new Map();
    const deletedFunctions = new Map();
    // map of previous name -> new name
    const renamedApps = new Map();
    const renamedFunctions = new Map();
    // 1. determine renamed apps using the firebase:name tag
    function checkRenamedProject(project, renamedCollection) {
        const renamedProject = isRenamed(project);
        if (renamedProject) {
            (0, utils_1.debugInfo)(`- ${renamedProject} has been renamed to ${project.name} `);
            renamedCollection.set(renamedProject, project);
        }
    }
    (0, utils_1.debugInfo)(`- checking for renamed apps & functions`);
    projects.firebaseAppProjects.forEach((project) => checkRenamedProject(project, renamedApps));
    projects.firebaseFunctionProjects.forEach((project) => checkRenamedProject(project, renamedFunctions));
    // 2. determine deleted functions
    // do this either by detecting a dependency
    (0, utils_1.debugInfo)(`- checking apps for deleted function deps`);
    projects.firebaseAppProjects.forEach((firebaseAppProject, firebaseAppProjectName) => {
        var _a;
        // check the implicitDependencies for each app first
        (_a = firebaseAppProject.implicitDependencies) === null || _a === void 0 ? void 0 : _a.map((dep) => {
            if (!projects.firebaseFunctionProjects.has(dep) &&
                !renamedFunctions.has(dep)) {
                (0, utils_1.debugInfo)(`- function ${dep} is a dep, but cannot be located so function is deleted`);
                deletedFunctions.set(dep, true);
            }
        });
        // functions may also be removed using nx g rm <function> --forceRemove
        // which removes the implicitDep from the app, but doesnt update the firebase config
        // so we need to check the firebase config too, and determine any projects that are in there
        // but not in the workspace and mark them as deleted
        const firebaseConfigName = configs.firebaseAppConfigs.get(firebaseAppProjectName);
        (0, utils_1.debugInfo)(`- checking config ${firebaseConfigName} to see if it contains a function that doesnt exist`);
        const config = configs.firebaseConfigs.get(firebaseConfigName);
        if (!config) {
            throw new Error(`Could not get firebase config '${firebaseConfigName}'`);
        }
        // ensure config functions is always an array, even if only 1 function
        // just in case user has modified this.
        if (!Array.isArray(config.functions)) {
            config.functions = [config.functions];
        }
        // remove deleted functions
        config.functions.forEach((func) => {
            const funcName = func.codebase;
            if (!projects.firebaseFunctionProjects.has(funcName) &&
                !renamedFunctions.has(funcName)) {
                (0, utils_1.debugInfo)(`- function ${funcName} is in config ${firebaseConfigName}, but cannot be located so function must be deleted`);
                deletedFunctions.set(funcName, true);
            }
        });
    });
    // nx g mv --project=source --destination=dstname
    // DOES update implicitDeps
    // Again, this means implicitDep wont be wrong, but we cant use it to detect renames
    // This is ok though because we detect renamed functions via tags
    // determine deleted apps
    (0, utils_1.debugInfo)(`- checking functions for deleted apps`);
    projects.firebaseFunctionProjects.forEach((firebaseFunctionProject, firebaseFunctionName) => {
        const { tagValue } = (0, tags_1.getFirebaseScopeFromTag)(firebaseFunctionProject, 'firebase:dep');
        if (!projects.firebaseAppProjects.has(tagValue) &&
            !renamedApps.has(tagValue)) {
            (0, utils_1.debugInfo)(`- function ${firebaseFunctionName} points to app ${tagValue} which doesnt exist, so app must be deleted`);
            deletedApps.set(tagValue, true);
        }
    });
    (0, utils_1.debugInfo)(`- deletedApps=${(0, utils_1.mapKeys)(deletedApps)}`);
    (0, utils_1.debugInfo)(`- deletedFunctions=${(0, utils_1.mapKeys)(deletedFunctions)}`);
    (0, utils_1.debugInfo)(`- renamedApps=${(0, utils_1.mapKeys)(renamedApps)}`);
    (0, utils_1.debugInfo)(`- renamedFunctions=${(0, utils_1.mapKeys)(renamedFunctions)}`);
    return {
        deletedApps,
        deletedFunctions,
        renamedApps,
        renamedFunctions,
    };
}
exports.getFirebaseChanges = getFirebaseChanges;
//# sourceMappingURL=firebase-changes.js.map