"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.syncGenerator = void 0;
const tslib_1 = require("tslib");
const devkit_1 = require("@nx/devkit");
const utils_1 = require("../../utils");
const init_1 = require("../init/init");
const lib_1 = require("./lib");
const types_1 = require("../../types");
const FUNCTIONS_DEPLOY_MATCHER = /(--only[ =]functions:)([^\s]+)/;
/**
 * Sync firebase workspace
 *
 */
function syncGenerator(tree, options) {
    return tslib_1.__awaiter(this, void 0, void 0, function* () {
        const tasks = [];
        // initialise plugin
        const initTask = yield (0, init_1.default)(tree, {});
        tasks.push(initTask);
        // otherwise, sync the workspace.
        // build lists of firebase apps & functions that have been deleted or renamed
        (0, lib_1.debugInfo)('- Syncing workspace');
        const workspace = (0, lib_1.getFirebaseWorkspace)(tree);
        devkit_1.logger.info(`This workspace has ${workspace.firebaseAppProjects.size} firebase apps and ${workspace.firebaseFunctionProjects.size} firebase functions\n\n`);
        // change the firebase project for an nx firebase app project
        if (options.project) {
            // --project option requires --app option to be specified
            if (!options.app) {
                throw new Error('--app not specified for --project option');
            }
            // validate app parameter
            const project = (0, devkit_1.readProjectConfiguration)(tree, options.app);
            if (!(0, lib_1.isFirebaseApp)(project)) {
                throw new Error(`Project '${options.app}' is not a Firebase application.`);
            }
            (0, lib_1.debugInfo)(`- changing project for ${options.app} to ${options.project}`);
            if ((0, lib_1.updateFirebaseAppDeployProject)(project.targets.firebase, options)) {
                (0, devkit_1.updateProjectConfiguration)(tree, options.app, project);
            }
            return;
        }
        /**
         * Nx automatically:
         * - updates implicitDependencies when projects are renamed with `nx g mv`
         * - deletes implicitDependencies when projects are deleted with `nx g rm`
         * So we do not have to consider these scenarios.
         */
        // 1. remove any orphaned firebase config files that no longer linked to an app
        workspace.firebaseConfigProjects.forEach((projectName, configName) => {
            if (projectName === types_1.CONFIG_NO_APP) {
                (0, lib_1.debugInfo)(`- found firebase config ${configName} with no app`);
                tree.delete(configName);
                // its ok to delete keys in forEach
                workspace.firebaseConfigProjects.delete(configName);
                workspace.firebaseConfigs.delete(configName);
                // dont need to sync firebaseAppConfigs since this app wont exist in there
                devkit_1.logger.info(`CHANGE Firebase config '${configName}' is no longer referenced by any firebase app, deleted`);
            }
        });
        // 2. rename firebase config files too if app is renamed
        workspace.renamedApps.forEach((project, oldProjectName) => {
            const configFileName = workspace.firebaseAppConfigs.get(project.name);
            // only rename config file if it is not firebase.json
            if (configFileName !== 'firebase.json') {
                const config = workspace.firebaseConfigs.get(configFileName);
                // create a copy of the firebase config with the renamed project name
                const newConfigFileName = `firebase.${project.name}.json`;
                (0, devkit_1.writeJson)(tree, newConfigFileName, config);
                // rewrite the --config=<configFileName> part of the firebase target command
                (0, utils_1.setFirebaseConfigFromCommand)(project, newConfigFileName);
                // write the updated project
                (0, devkit_1.updateProjectConfiguration)(tree, project.name, project);
                // delete the original config from the workspace
                tree.delete(configFileName);
                // rewrite the workspace to the newly renamed config file
                workspace.firebaseConfigs.delete(configFileName);
                workspace.firebaseAppConfigs.delete(project.name);
                workspace.firebaseConfigProjects.delete(configFileName);
                workspace.firebaseConfigs.set(newConfigFileName, config);
                workspace.firebaseAppConfigs.set(project.name, newConfigFileName);
                workspace.firebaseConfigProjects.set(newConfigFileName, project.name);
                devkit_1.logger.info(`CHANGE Firebase app '${oldProjectName}' was renamed to '${project.name}', renamed config file to '${newConfigFileName}'`);
            }
        });
        // 3. update the firebase:name tag for renamed apps
        workspace.renamedApps.forEach((project, oldName) => {
            (0, lib_1.updateFirebaseProjectNameTag)(tree, project);
            devkit_1.logger.info(`CHANGE Firebase app '${oldName}' was renamed to '${project.name}', updated firebase:name tag`);
            // we also need to update nx:run-commands in the renamed projects for various targets
            // test target
            (0, lib_1.renameCommandForTarget)(tree, project, 'test', `tag:firebase:dep:${oldName}`, `tag:firebase:dep:${project.name}`);
            // lint target
            (0, lib_1.renameCommandForTarget)(tree, project, 'lint', `tag:firebase:dep:${oldName}`, `tag:firebase:dep:${project.name}`);
            // watch target
            (0, lib_1.renameCommandForTarget)(tree, project, 'watch', `tag:firebase:dep:${oldName}`, `tag:firebase:dep:${project.name}`);
            // serve target
            (0, lib_1.renameCommandForTarget)(tree, project, 'serve', `${oldName}:`, `${project.name}:`);
            // deploy target
            (0, lib_1.renameCommandForTarget)(tree, project, 'deploy', `${oldName}:`, `${project.name}:`);
            // getconfig target
            (0, lib_1.renameCommandForTarget)(tree, project, 'getconfig', `${oldName}:`, `${project.name}:`);
            (0, lib_1.renameCommandForTarget)(tree, project, 'getconfig', `/${oldName}/`, `/${project.name}/`);
            // killports target
            (0, lib_1.renameCommandForTarget)(tree, project, 'killports', `${oldName}:`, `${project.name}:`);
            // emulate target
            (0, lib_1.renameCommandForTarget)(tree, project, 'emulate', `/${oldName}/`, `/${project.name}/`);
            (0, lib_1.renameCommandForTarget)(tree, project, 'emulate', `${oldName}:`, `${project.name}:`);
            devkit_1.logger.info(`CHANGE Firebase app '${oldName}' was renamed to '${project.name}', updated targets`);
        });
        // 4. update the firebase:dep tag for functions linked to renamed apps
        workspace.firebaseFunctionProjects.forEach((project, name) => {
            const { tagValue, tagIndex } = (0, lib_1.getFirebaseScopeFromTag)(project, 'firebase:dep');
            if (workspace.renamedApps.has(tagValue)) {
                // update the firebase:dep tag
                const renamedApp = workspace.renamedApps.get(tagValue);
                project.tags[tagIndex] = `firebase:dep:${renamedApp.name}`;
                devkit_1.logger.info(`CHANGE Firebase app '${tagValue}' was renamed to '${renamedApp.name}', updated firebase:dep tag in firebase function '${name}'`);
                // update the environment assets glob also
                const functionAssets = project.targets.build.options.assets;
                for (const asset of functionAssets) {
                    if (typeof asset === 'object') {
                        if (asset.input) {
                            asset.input = (0, devkit_1.joinPathFragments)(renamedApp.root, 'environment');
                            devkit_1.logger.info(`CHANGE Firebase app '${tagValue}' was renamed to '${renamedApp.name}', updated environment assets path in firebase function '${name}'`);
                        }
                    }
                }
                // update the deploy command in functions where app has been renamed
                const deployCommand = project.targets.deploy.options.command;
                project.targets.deploy.options.command = deployCommand.replace(tagValue, renamedApp.name);
                devkit_1.logger.info(`CHANGE Firebase app '${tagValue}' was renamed to '${renamedApp.name}', updated firebase deploy command in firebase function '${name}'`);
                // update the function project config
                (0, devkit_1.updateProjectConfiguration)(tree, project.name, project);
            }
            else {
                if (workspace.deletedApps.has(tagValue) ||
                    !workspace.firebaseAppProjects.has(tagValue)) {
                    devkit_1.logger.warn(`CHANGE Firebase app '${tagValue}' was deleted, firebase:dep tag for firebase function '${name}' is no longer linked to a Firebase app.`);
                }
            }
        });
        // 5. update the firebase:name tag for renamed functions
        workspace.renamedFunctions.forEach((project, oldName) => {
            (0, lib_1.updateFirebaseProjectNameTag)(tree, project);
            devkit_1.logger.info(`CHANGE Firebase function '${oldName}' was renamed to '${project.name}', updated firebase:name tag`);
        });
        // 6. update the deploy command for renamed functions
        workspace.renamedFunctions.forEach((project, oldName) => {
            const deployCommand = project.targets.deploy.options.command;
            project.targets.deploy.options.command = deployCommand.replace(FUNCTIONS_DEPLOY_MATCHER, '$1' + project.name);
            devkit_1.logger.info(`CHANGE Firebase function '${oldName}' was renamed to '${project.name}', updated deploy target to '--only=functions:${project.name}'`);
            (0, devkit_1.updateProjectConfiguration)(tree, project.name, project);
        });
        // 7. sync firebase configs for deleted & renamed functions
        workspace.firebaseConfigs.forEach((config, configFileName) => {
            let configUpdated = false;
            const functions = config.functions;
            const updatedFunctions = [];
            // remove deleted functions
            functions.forEach((func) => {
                // remove functions where codebase is linked to a now deleted firebase function project
                if (workspace.deletedFunctions.has(func.codebase)) {
                    devkit_1.logger.info(`CHANGE Firebase function '${func.codebase}' was deleted, removing function codebase from '${configFileName}'`);
                    configUpdated = true;
                }
                else {
                    // rename function codebase if linked to a renamed firebase function project
                    const codebase = func.codebase;
                    (0, lib_1.debugInfo)(`- checking if codebase '${codebase}' is renamed`);
                    if (workspace.renamedFunctions.has(codebase)) {
                        // change name of codebase
                        const project = workspace.renamedFunctions.get(codebase);
                        (0, lib_1.debugInfo)(`- codebase '${codebase}' is renamed to ${project.name}`);
                        func.codebase = project.name;
                        // change source dir
                        func.source = project.targets.build.options.outputPath;
                        devkit_1.logger.info(`CHANGE Firebase function '${codebase}' was renamed to '${project.name}', updated codebase in '${configFileName}'`);
                        configUpdated = true;
                    }
                    updatedFunctions.push(func);
                }
            });
            if (configUpdated) {
                config.functions = updatedFunctions;
                config.functions.sort((a, b) => {
                    return a.codebase < b.codebase ? -1 : a.codebase > b.codebase ? 1 : 0;
                });
                (0, devkit_1.writeJson)(tree, configFileName, config);
            }
        });
        // 8. sync firebase configs for rules & hosting for renamed apps
        workspace.renamedApps.forEach((project, oldProjectName) => {
            const configFileName = workspace.firebaseAppConfigs.get(project.name);
            const config = workspace.firebaseConfigs.get(configFileName);
            // update config rules that are in the firebase app
            const databaseRules = (0, devkit_1.joinPathFragments)(project.root, 'database.rules.json');
            if (config.database && config.database.rules !== databaseRules) {
                config.database.rules = databaseRules;
                devkit_1.logger.info(`CHANGE Firebase app '${oldProjectName}' was renamed to '${project.name}', updated database rules in '${configFileName}'`);
            }
            const firestoreRules = (0, devkit_1.joinPathFragments)(project.root, 'firestore.rules');
            if (config.firestore && config.firestore.rules !== firestoreRules) {
                config.firestore.rules = firestoreRules;
                devkit_1.logger.info(`CHANGE Firebase app '${oldProjectName}' was renamed to '${project.name}', updated firestore rules in '${configFileName}'`);
            }
            const firestoreIndexes = (0, devkit_1.joinPathFragments)(project.root, 'firestore.indexes.json');
            if (config.firestore && config.firestore.indexes !== firestoreIndexes) {
                config.firestore.indexes = firestoreIndexes;
                devkit_1.logger.info(`CHANGE Firebase app '${oldProjectName}' was renamed to '${project.name}', updated firestore indexes in '${configFileName}'`);
            }
            const storageRules = (0, devkit_1.joinPathFragments)(project.root, 'storage.rules');
            if (config.storage && config.storage.rules !== storageRules) {
                config.storage.rules = storageRules;
                devkit_1.logger.info(`CHANGE Firebase app '${oldProjectName}' was renamed to '${project.name}', updated storage rules in '${configFileName}'`);
            }
            (0, devkit_1.writeJson)(tree, configFileName, config);
        });
        // 9. validate hosting rules match a project
        workspace.firebaseConfigs.forEach((config, configFileName) => {
            const projects = (0, devkit_1.getProjects)(tree);
            if (!Array.isArray(config.hosting)) {
                config.hosting = [config.hosting];
            }
            config.hosting.forEach((host) => {
                let isValid = false;
                projects.forEach((nxProject) => {
                    if (host.public.includes(nxProject.root)) {
                        isValid = true;
                    }
                });
                if (!isValid) {
                    devkit_1.logger.warn(`WARNING: Can't match hosting target with public dir '${host.public}' in '${configFileName}' to a project in this workspace. Is it configured correctly?`);
                }
            });
        });
        // if user deletes a project that was linked to firebase.json config but there
        // are other firebase apps in the workspace, we need to inform user about
        // this, since we dont have a way
        // other firebase apps in the workspace, we'll just advise
        if (!tree.exists('firebase.json') && workspace.firebaseAppProjects.size) {
            devkit_1.logger.warn(`None of the Firebase apps in this workspace use 'firebase.json' as their config. Firebase CLI may not work as expected. This can be fixed by renaming the config for one of your firebase projects to 'firebase.json'.`);
        }
        return (0, devkit_1.runTasksInSerial)(...tasks);
    });
}
exports.syncGenerator = syncGenerator;
exports.default = syncGenerator;
//# sourceMappingURL=sync.js.map