"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.setFirebaseConfigFromCommand = exports.getFirebaseConfigFromProject = exports.getFirebaseConfigFromCommand = void 0;
const FIREBASE_TARGET_CONFIG_MATCHER = /(--config[ =])([^\s]+)/;
/**
 * Return the config file from the provided firebase target command
 * This can be used to parse commands in additional configurations
 * @param project
 * @param command
 * @returns
 */
function getFirebaseConfigFromCommand(tree, project, command) {
    const match = command.match(FIREBASE_TARGET_CONFIG_MATCHER);
    if (match && match.length === 3) {
        const configName = match[2];
        // check the config we've parsed actually resolves to a firebase config file in the workspace
        if (!tree.exists(configName)) {
            throw new Error(`Firebase app project ${project.name} is using a firebase config file ${configName} that does not exist in the workspace.`);
        }
        return configName;
    }
    throw new Error(`Firebase app project ${project.name} does not have --config set in its 'firebase' target.`);
}
exports.getFirebaseConfigFromCommand = getFirebaseConfigFromCommand;
/**
 * Return the config file used by the `firebase` target command of the provided firebase app project
 * @param command
 * @param project
 * @param firebaseConfigs
 * @returns
 */
function getFirebaseConfigFromProject(tree, project) {
    return getFirebaseConfigFromCommand(tree, project, project.targets.firebase.options.command);
}
exports.getFirebaseConfigFromProject = getFirebaseConfigFromProject;
/**
 * Modify the config file used by the `firebase` target command of the provided firebase app project
 * @param project
 * @param configFileName
 */
function setFirebaseConfigFromCommand(project, configFileName) {
    // we've already checked that firebase target exists when setting up workspace
    const firebaseTarget = project.targets.firebase;
    firebaseTarget.options.command = firebaseTarget.options.command.replace(FIREBASE_TARGET_CONFIG_MATCHER, '$1' + configFileName);
    // do this for all other configurations on this target too
    const configurations = firebaseTarget.configurations;
    for (const configuration in configurations) {
        configurations[configuration].command = configurations[configuration].command.replace(FIREBASE_TARGET_CONFIG_MATCHER, '$1' + configFileName);
    }
}
exports.setFirebaseConfigFromCommand = setFirebaseConfigFromCommand;
//# sourceMappingURL=firebase-config.js.map